/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/*  (C) Copyright SAP AG, Walldorf  1997 -1999                        */
/*                                                                    */
/*                                                                    */
/*  This is a sample implementation of the auxiliary functions.       */
/*                                                                    */
/*  You may use parts of this source or the complete source           */
/*  to facilitate the implementation of your security toolkit.        */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  ssfxxaux.c                                                        */
/*                                                                    */
/*  ssfxxaux.c (explanation of the file name)                         */
/*  ==========                                                        */
/*                                                                    */
/*  This file contains the function implementations for the           */
/*  auxiliary functions of the SSF ("Secure Store & Forward") API.    */
/*  The auxiliary functions can be used by applications using the     */
/*  SSF API to work with the SSF list structures which have to be     */
/*  supplied as input parameters or which are returned as output      */
/*  parameters. In particular, the auxiliary functions deal with      */
/*  security information lists for signers and recipients including   */
/*  result information. The following auxiliary functions are         */
/*  provided:                                                         */
/*                                                                    */
/*      SsfNEWSigRcpSsfInfo     create new signer/recipient info      */
/*      SsfDELSigRcpSsfInfo     delete signer/Recipient info          */
/*      SsfINSSigRcpSsfInfo     insert signer/recipient info into list*/
/*      SsfDELSigRcpSsfInfoList delete signer/recipient info list     */
/*                                                                    */
/*                                                                    */
/*  Author:    J. M. SCHNEIDER                                        */
/*  Reviewer:                                                         */
/*  Version:   1.32                                                   */
/*                                                                    */
/*  Description:                                                      */
/*    The function implementations contained in this file realize the */
/*    auxiliary functions for using the SSF API.                      */
/*                                                                    */
/*  Prerequisites:                                                    */
/*    - programming environment                                       */
/*    - system libraries                                              */
/*    - standard conformance: platform-independent C code             */
/*      (e. g. Standard C, POSIX, X/OPEN)                             */
/*    - others                                                        */
/*                                                                    */
/**********************************************************************/

/* Add or remove C standard headers */
#include <stdio.h>
#include <assert.h>
/* ... */

/**********************************************************************/
/*                                                                    */
/*  Import interface                                                  */
/*    - include files of other projects                               */
/*                                                                    */
/**********************************************************************/

/**********************************************************************/
/*                                                                    */
/*  Export interface                                                  */
/*    - header files of this project                                  */
/*                                                                    */
/**********************************************************************/

#include "ssfxxapi.h"

/**********************************************************************/
/*                                                                    */
/*  Public data definition                                            */
/*                                                                    */
/**********************************************************************/


/* SSF auxiliary functions for signer security information lists -----*/

/**********************************************************/
/*                                                        */
/*  Function:  SsfNEWSigRcpSsfInfo                        */
/*                                                        */
/*  Author:    J. M. SCHNEIDER                            */
/*  Version:   1.32                                       */
/*                                                        */
/* Creates a new signer/recipient SSF information data    */
/* structure by allocating and initializing memory.       */
/* Returns a pointer to the new signer/recipient          */
/* information data.                                      */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* SsfCharstring  strSigRcpId           sig./recip. name  */
/* SsfCharlen     strSigRcpIdL          length of above   */
/* SsfCharstring  strSigRcpReserved     resrved for furtur*/
/*                                      (if any, can be   */
/*                                      empty)            */
/* SsfCharlen     strSigRcpReservedL    length of above   */
/* SsfCharstring  strSigRcpProfile      sig/rcp secinfo   */
/*                                      profile name      */
/*                                      (if any, may be   */
/*                                      empty)            */
/* SsfCharlen     strSigRcpProfileL     length of above   */
/* SsfCharstring  strSigRcpPassword     password for      */
/*                                      sig/rcp secinfo   */
/*                                      (if any, may be   */
/*                                      empty)            */
/* SsfCharlen     strSigRcpPasswordL    length of above   */
/* SAP_UINT       uResult               result value      */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* PtrSigRcpSsf                         ptrSigRcpSsfInfo  */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOMEMORY      insuff. main memory output data  */
/* SSF_AUX_NOSIGRCPID    sig/rcp id is empty              */
/*                                                        */
/**********************************************************/

SAPRETURN APIENTRY SsfNEWSigRcpSsfInfo(
			const SsfCharstring		strSigRcpId,
			SsfCharlen				strSigRcpIdL,
			const SsfCharstring		strSigRcpReserved,
			SsfCharlen				strSigRcpReservedL,
			const SsfCharstring		strSigRcpProfile,
			SsfCharlen				strSigRcpProfileL,
			const SsfCharstring		strSigRcpPassword,
			SsfCharlen				strSigRcpPasswordL,
			SAP_UINT				uResult,
			PtrSigRcpSsfInformation * ptrSigRcpSsfInfo)
{
	SAPRETURN						rc = -1;
	PtrSigRcpSsfInformation	        ptrNew = NULL;
    SAP_BOOL                        bOutOfMem = FALSE;

	SsfCharstring  					lstrSigRcpId = NULL;
	SsfCharstring	 				lstrSigRcpReserved = NULL;
	SsfCharstring	 				lstrSigRcpProfile = NULL;
	SsfCharstring	 				lstrSigRcpPassword = NULL;

	/* check id length */
	if ( (strSigRcpIdL > 0) &&
         (strSigRcpReservedL >= 0) &&
         (strSigRcpProfileL >= 0) &&
         (strSigRcpPasswordL >= 0) ) {

		/* try to allocate memory for one SigRcpSsfInformation */
		ptrNew = (PtrSigRcpSsfInformation) malloc(sizeof *ptrNew);

		if (ptrNew != NULL) { /* new memory is available */

			/* try allocate memory for copying input parameters */

			lstrSigRcpId = (SsfCharstring) malloc (strSigRcpIdL);
            if (NULL == lstrSigRcpId) bOutOfMem = TRUE;

            if (strSigRcpReservedL > 0) {
				lstrSigRcpReserved = (SsfCharstring) malloc (strSigRcpReservedL);
				if (NULL == lstrSigRcpReserved) bOutOfMem = TRUE;
			}

            if (strSigRcpProfileL > 0) {
				lstrSigRcpProfile = (SsfCharstring) malloc (strSigRcpProfileL);
				if (NULL == lstrSigRcpProfile) bOutOfMem = TRUE;
            }

            if  (strSigRcpPasswordL > 0) {
				lstrSigRcpPassword = (SsfCharstring) malloc (strSigRcpPasswordL);
				if (NULL == lstrSigRcpPassword) bOutOfMem = TRUE;
            }

			if (FALSE == bOutOfMem) {
				/* memory is available, do copy */
				memcpy(lstrSigRcpId, strSigRcpId, strSigRcpIdL);
				ptrNew->strSigRcpId = lstrSigRcpId;
				ptrNew->strSigRcpIdL = strSigRcpIdL;

				memcpy(lstrSigRcpReserved, strSigRcpReserved, strSigRcpReservedL);
				ptrNew->strSigRcpReserved = lstrSigRcpReserved;
				ptrNew->strSigRcpReservedL = strSigRcpReservedL;

				memcpy(lstrSigRcpProfile, strSigRcpProfile, strSigRcpProfileL);
				ptrNew->strSigRcpProfile = lstrSigRcpProfile;
				ptrNew->strSigRcpProfileL = strSigRcpProfileL;

				memcpy(lstrSigRcpPassword, strSigRcpPassword, strSigRcpPasswordL);
				ptrNew->strSigRcpPassword = lstrSigRcpPassword;
				ptrNew->strSigRcpPasswordL = strSigRcpPasswordL;

                ptrNew->uResult = uResult;

				/* set successful return code */

				rc = SSF_AUX_OK;
			} else { /* no new memory available */
				if (NULL != lstrSigRcpId) free (lstrSigRcpId);
				if (NULL != lstrSigRcpReserved) free (lstrSigRcpReserved);
				if (NULL != lstrSigRcpProfile) free (lstrSigRcpProfile);
				if (NULL != lstrSigRcpPassword) free (lstrSigRcpPassword);

				free (ptrNew);
				ptrNew = NULL;
				rc = SSF_AUX_NOMEMORY;
			}
		} else {  /* no memory available */
			rc = SSF_AUX_NOMEMORY;
		}
	} else { /* id is not ok */
		rc = SSF_AUX_NOSIGRCPID;
	}

	*ptrSigRcpSsfInfo = ptrNew;
	return (rc);

} /* SsfNEWSigRcpSsfInfo */



/**********************************************************/
/*                                                        */
/*  Function:  SsfDELSigRcpSsfInfo                        */
/*                                                        */
/*  Author:    J. M. SCHNEIDER                            */
/*  Version:   1.32                                       */
/*                                                        */
/* Deletes a signer/recipient information data structure  */
/* and frees all associated memory (including contents).  */
/* Returns a NULL pointer.                                */
/*                                                        */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* PtrSigRcpSsfInformation         ptrSigRcpSsfInfo       */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOSIGRCPINFO  signer/recipient information     */
/*                       data structure is empty          */
/*                                                        */
/**********************************************************/

SAPRETURN APIENTRY SsfDELSigRcpSsfInfo	(
			PtrSigRcpSsfInformation  * ptrSigRcpSsfInfo)
{
	/* function body */

	PtrSigRcpSsfInformation		tbd;

	tbd = *ptrSigRcpSsfInfo;

	if ( NULL != tbd ) {

		if (NULL != tbd->strSigRcpId)
            free ( tbd->strSigRcpId );
		if (NULL != tbd->strSigRcpReserved)
            free ( tbd->strSigRcpReserved );
		if (NULL != tbd->strSigRcpProfile)
            free ( tbd->strSigRcpProfile );
        if (NULL != tbd->strSigRcpPassword) {
    		/* destroy password information in memory */
	    	memset(tbd->strSigRcpPassword,' ',tbd->strSigRcpPasswordL);
		    free ( tbd->strSigRcpPassword );
        }

		free ( tbd );

		*ptrSigRcpSsfInfo = NULL;

		return ( SSF_AUX_OK );
	}
	else {

		return ( SSF_AUX_NOSIGRCPINFO );
	}

} /* SsfDELSigRcpSsfInfo */


/**********************************************************/
/*                                                        */
/*  Function:  SsfINSSigRcpSsfInfo                        */
/*                                                        */
/*  Author:    J. M. SCHNEIDER                            */
/*  Version:   1.32                                       */
/*                                                        */
/* Creates a new signer/recipient information list element*/
/* and inserts a given signer/recipient information data  */
/* structure at the end of a given signer/recipient       */
/* information list.                                      */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* PtrSigRcpSsfInformation        ptrSigRcpSsfInfo        */
/*                                                        */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* SigRcpSsfInformationList       sigRcpSsfInfoList       */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOMEMORY      insuff. main memory output data  */
/* SSF_AUX_NOSIGRCPINFO  signer/recipient information     */
/*                       data structure is empty          */
/*                                                        */
/**********************************************************/

SAPRETURN APIENTRY SsfINSSigRcpSsfInfo	(
			const PtrSigRcpSsfInformation  ptrSigRcpSsfInfo,
			SigRcpSsfInformationList * sigRcpSsfInfoList )
{
	/* function body */

	SAPRETURN					rc = -1;
	SigRcpSsfInformationList	ptrNewElem = NULL;
	SigRcpSsfInformationList    ptrHelp = NULL;

	/* check if element to be inserted is ok */
	if (ptrSigRcpSsfInfo != NULL) /* element is ok */
	{
		/* try to allocate memory for new element */
		ptrNewElem = (SigRcpSsfInformationList)
            malloc(sizeof *ptrNewElem);

		if (ptrNewElem != NULL) /* memory available */
		{
			/* assign information value pointers for new element */
			ptrNewElem->ptrSigRcp = ptrSigRcpSsfInfo;
			ptrNewElem->ptrNextSigRcp = NULL;

			/* insert new element in list */

			/* check if list is empty */
			if (*sigRcpSsfInfoList == NULL) /* list is empty */
			{
				*sigRcpSsfInfoList = ptrNewElem;
			}
			else /* list is not empty */
			{
				/* find end of list */
				for (	ptrHelp = *sigRcpSsfInfoList;
						(ptrHelp->ptrNextSigRcp != NULL);
						ptrHelp = ptrHelp->ptrNextSigRcp )
						{
							;
						} /* endfor */

				/* insert at end of list */
				ptrHelp->ptrNextSigRcp = ptrNewElem;

			} /* endif */

			/* set successful return code */
			rc = SSF_AUX_OK;
		}
		else /* no new memory available */
		{
			rc = SSF_AUX_NOMEMORY;
		} /* endif */
	}
	else /* element to be inserted is not ok */
	{
		rc = SSF_AUX_NOSIGRCPINFO;
	} /* endif */

	return (rc);

} /* SsfINSSigRcpSsfInfo */



/**********************************************************/
/*                                                        */
/*  Function:  SsfDELSigRcpSsfInfoList                    */
/*                                                        */
/*  Author:    J. M. SCHNEIDER                            */
/*  Version:   1.32                                       */
/*                                                        */
/* Deletes a signer/recipient information list element by */
/* element and frees the corresponding memory.            */
/* (including content structure)                          */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* SigRcpSsfInformationList       sigRcpSsfInfoList       */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK               operation successful          */
/* SSF_AUX_NOSIGRCPINFOLIST signer/recipient info list is */
/*                          empty                         */
/*                                                        */
/**********************************************************/

SAPRETURN APIENTRY SsfDELSigRcpSsfInfoList	(
			SigRcpSsfInformationList * sigRcpSsfInfoList )
{
	/* function body */

	SAPRETURN			       rc = -1;
	SigRcpSsfInformationList   ptrHelp1 = NULL;
	SigRcpSsfInformationList   ptrHelp2 = NULL;

	/* check if list is not empty */
	if (*sigRcpSsfInfoList != NULL) /* list is not empty */
	{
		/* free list element by element */
		ptrHelp1 = *sigRcpSsfInfoList;
		ptrHelp2 = *sigRcpSsfInfoList;

		while (ptrHelp2 != NULL)
		{
			ptrHelp2 = ptrHelp2->ptrNextSigRcp;
			SsfDELSigRcpSsfInfo ( &(ptrHelp1->ptrSigRcp) );
   			free (ptrHelp1);
			ptrHelp1 = ptrHelp2;
		}

		/* set output parameter and successful return code */

		*sigRcpSsfInfoList = NULL;
		rc = SSF_AUX_OK;
	}
	else /* list is empty */
	{
		rc = SSF_AUX_NOSIGRCPINFOLIST;
	} /* endif */

	return (rc);

} /* SsfDELSigRcpSsfInfoList */


/**********************************************************/
/*                                                        */
/*  Function:  SsfPRISigRcpSsfInfo                        */
/*                                                        */
/*  Author:    M. FRIEDRICH                               */
/*  Version:   1.32                                       */
/*                                                        */
/* Prints  a signer/recipient information data structure  */
/* to memory                                              */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* PtrSigRcpSsfInformation        ptrSigRcpSsfInfo        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrOutputData        signer/recipient information     */
/* ostrOutputDataL       length of above                  */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOMEMORY      no more memory for output data   */
/* SSF_AUX_NOSIGRCPINFO  signer/recipient information     */
/*                       data structure is empty          */
/*                                                        */
/**********************************************************/

SAPRETURN APIENTRY SsfPRISigRcpSsfInfo(
				PtrSigRcpSsfInformation ptrSigRcpSsfInfo,
				SsfOctetstring		* outputData,
				SsfOctetlen			* outputDataL)
{

	SsfOctetstring				lsOutputData=NULL;
	SsfOctetlen					lsOutputDataL=0;
	int							j;


	/* check if info is not empty */
	if (ptrSigRcpSsfInfo == NULL) /* info is empty */
		return (SSF_AUX_NOSIGRCPINFO);

	/* calculate size of text */
	lsOutputDataL = sizeof("Signer/Recipient Security Information :\n")+2;
	if (ptrSigRcpSsfInfo->strSigRcpIdL > 0)
		lsOutputDataL += 25 + ptrSigRcpSsfInfo->strSigRcpIdL;
	if (ptrSigRcpSsfInfo->strSigRcpReservedL > 0)
		lsOutputDataL += 25 + ptrSigRcpSsfInfo->strSigRcpReservedL;
	if (ptrSigRcpSsfInfo->strSigRcpProfileL > 0)
		lsOutputDataL += 25 + ptrSigRcpSsfInfo->strSigRcpProfileL;
	lsOutputDataL += sizeof("\n\nSigner/Recipient Result : ");
	lsOutputDataL += sizeof("UNKNOWN RESULT ???\n"); /* largest uResult text */
	lsOutputDataL += 2;

	lsOutputData = (SsfOctetstring) malloc (lsOutputDataL);
	if (lsOutputData == NULL)
		return (SSF_AUX_NOMEMORY);

	/* now print the text to memory */
	j = sprintf (lsOutputData, "Signer/Recipient Security Information :\n");

	/* max width is 14+strSigRcpIdL+width_of_strSigRcpIdL (should be <10) */
    if (ptrSigRcpSsfInfo->strSigRcpIdL > 0) {
		j += sprintf (lsOutputData+j, "Id        : %.*s (%d)\n",
			    ptrSigRcpSsfInfo->strSigRcpIdL,
				ptrSigRcpSsfInfo->strSigRcpId,
				ptrSigRcpSsfInfo->strSigRcpIdL);
    }

    if (ptrSigRcpSsfInfo->strSigRcpReservedL > 0) {
		j+= sprintf (lsOutputData+j, "Reserved  : %.*s (%d)\n",
			    ptrSigRcpSsfInfo->strSigRcpReservedL,
				ptrSigRcpSsfInfo->strSigRcpReserved,
				ptrSigRcpSsfInfo->strSigRcpReservedL);
    }

    if (ptrSigRcpSsfInfo->strSigRcpProfileL > 0) {
		j += sprintf (lsOutputData+j, "Profile   : %.*s (%d)\n",
			    ptrSigRcpSsfInfo->strSigRcpProfileL,
				ptrSigRcpSsfInfo->strSigRcpProfile,
				ptrSigRcpSsfInfo->strSigRcpProfileL);
    }

	j += sprintf (lsOutputData+j, "\n\nSigner/Recipient Result : ");

	switch (ptrSigRcpSsfInfo->uResult) {
	case SSF_API_SIGNER_OK:
    /* equals case SSF_API_RECIPIENT_OK: */
		j += sprintf (lsOutputData+j, "OK\n");
		break;
	case SSF_API_INVALID_NAMESPACE:
		j += sprintf (lsOutputData+j, "INVALID_NAMESPACE\n");
		break;
	case SSF_API_UNKNOWN_SIGNER:
    /* equals case SSF_API_UNKNOWN_RECIPIENT: */
		j += sprintf (lsOutputData+j, "UNKNOWN\n");
		break;
	case SSF_API_UNKNOWN_PROFILE:
		j += sprintf (lsOutputData+j, "UNKNOWN_PROFILE\n");
		break;
	case SSF_API_INVALID_PROFILE:
		j += sprintf (lsOutputData+j, "INVALID_PROFILE\n");
		break;
	case SSF_API_INVALID_PASSWORD:
		j += sprintf (lsOutputData+j, "INVALID_PASSWORD\n");
		break;
	case SSF_API_NOCERTIFICATE:
		j += sprintf (lsOutputData+j, "NOCERTIFICATE\n");
		break;
	case SSF_API_SIGNER_NOT_OK:
    /* equals case SSF_API_RECIPIENT_NOT_OK: */
		j += sprintf (lsOutputData+j, "NOT_OK\n");
		break;
    case SSF_API_UNDEFINED_RESULT:
		j += sprintf (lsOutputData+j, "UNDEFINED\n");
		break;
	default:
		j += sprintf (lsOutputData+j, "UNKNOWN RESULT ???\n");
		break;
	} /* endswitch */

	j += sprintf (lsOutputData+j, "\n\n");

	assert(j<lsOutputDataL);

	/* null-terminate string - isn't necessary but might be useful */
	*(lsOutputData+j) = 0;

	/* set successful return code */
	*outputData = lsOutputData;
	*outputDataL = j;
	return (SSF_AUX_OK);

} /* SsfPRISigRcpSsfInfo */


/**********************************************************/
/*                                                        */
/*  Function:  SsfPRISigRcpSsfInfoList                    */
/*                                                        */
/*  Author:    M. FRIEDRICH                               */
/*  Version:   1.32                                       */
/*                                                        */
/* Prints  a signer/recipient information list element by */
/* element to memory                                      */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* SigRcpSsfInformationList       sigRcpSsfInfoList       */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrOutputData        list information                 */
/* ostrOutputDataL       length of above                  */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOMEMORY      no more memory for output data   */
/* SSF_AUX_NOSIGRCPINFOLIST signer/recipient info list is */
/*                          empty                         */
/*                                                        */
/**********************************************************/


SAPRETURN APIENTRY SsfPRISigRcpSsfInfoList(
				SigRcpSsfInformationList sigRcpSsfInfoList,
				SsfOctetstring		* outputData,
				SsfOctetlen			* outputDataL)
{
	SsfOctetstring				lsOutputData=NULL;
	SsfOctetlen					lsOutputDataL=0;
	int							j;

	SAP_UINT                    count = 0;		 /* count signers in list */
	SigRcpSsfInformationList	ptrHelp = NULL;	 /* run for list */


	/* check if list is not empty */
	if (sigRcpSsfInfoList == NULL) /* list is empty */
		return(SSF_AUX_NOSIGRCPINFOLIST);

	/* print output header */
	lsOutputDataL = sizeof("SIGNER/RECIPIENT SECURITY INFORMATION LIST :\n\n");
	lsOutputDataL += 2;

	lsOutputData = (SsfOctetstring) malloc (lsOutputDataL);
	if (lsOutputData == NULL)
		return (SSF_AUX_NOMEMORY);

	j = sprintf (lsOutputData, "SIGNER/RECIPIENT SECURITY INFORMATION LIST :\n\n");

	/* traverse list and print information element by element */

	/* initialize signer counter */
	count = 1;
	/* start loop */
	for (	ptrHelp = sigRcpSsfInfoList;
			(ptrHelp != NULL);
			ptrHelp = ptrHelp->ptrNextSigRcp )
			{
				SsfOctetstring tmpOutputData=NULL;
				SAP_INT tmpOutputDataL=0;
				SsfOctetstring newOutputData=NULL;
				SAP_INT newOutputDataL=0;
				int rc;

				rc = SsfPRISigRcpSsfInfo (ptrHelp->ptrSigRcp,&tmpOutputData,&tmpOutputDataL);
				if (rc != SSF_AUX_OK) { /* must be SSF_AUX_NOMEMORY */
					free(lsOutputData);
					return (rc);
				}

				/* calculate new size */
				newOutputDataL = lsOutputDataL + tmpOutputDataL + 14 + 20 + 14 + 5;
				/* 14+20+14: lines below, max 5 chars for the number */

				newOutputData = (SsfOctetstring) malloc (newOutputDataL);
				if (newOutputData == NULL) {
					free(lsOutputData);
					return (SSF_AUX_NOMEMORY);
				}
				strncpy(newOutputData, lsOutputData, j);

				free(lsOutputData);
				lsOutputData=newOutputData;
				lsOutputDataL=newOutputDataL;

				j += sprintf(lsOutputData+j, "=============\n");
				j += sprintf(lsOutputData+j, "Signer/Recipient : %d\n", count++);
				j += sprintf(lsOutputData+j, "=============\n");

				strncpy(lsOutputData+j, tmpOutputData, tmpOutputDataL);
				j += tmpOutputDataL;

				free(tmpOutputData); /* or: SsfDELSsfOctetstring */

				assert(j<lsOutputDataL);
			} /* endfor */

	assert(j<lsOutputDataL);

	/* null-terminate string - isn't necessary but might be useful */
	*(lsOutputData+j) = 0;

	/* set successful return code */
	*outputData = lsOutputData;
	*outputDataL = j;
	return (SSF_AUX_OK);


} /* SsfPRISigRcpSsfInfoList */

