/* @(#) $Id: //bas/BIN/src/sec/ssf/ssfxxapi.h#5 $ SAP*/
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/*  (C) Copyright SAP AG, Walldorf  1996 - 1999                       */
/*                                                                    */
/*                                                                    */
/*  ssfxxapi.h     C API for SSF (Secure Store & Forward)             */
/*  ==========                                                        */
/*                                                                    */
/* This header file contains all declarations for the SSF API.        */
/* (Macros, constants and types can be found in ssfxxall.h)           */
/*                                                                    */
/* The SSF ("Secure Store & Forward") API provides services to        */
/* protect SAP R/3 data and documents while such data is being stored */
/* on insecure media or is in transit over insecure communication     */
/* channels.                                                          */
/*                                                                    */
/* SSF provides data integrity, authentication and non-repudiation    */
/* services using digital signatures, as well as confidentiality      */
/* through the use of digital envelopes and encryption. Through the   */
/* SSF API security toolkits from external vendors are integrated     */
/* with SAP R/3.                                                      */
/*                                                                    */
/*                                                                    */
/* The following functions are provided in the "all-in-one buffer-    */
/* based processing (abp)" mode (in Version 1.1x):                    */
/*                                                                    */
/*    SsfVersion         return version info of SSF library           */
/*    SsfSign            create digital signature(s) for data         */
/*    SsfVerify          verify digital signatures for data           */
/*    SsfEnvelope        encrypt data for recipient(s)                */
/*    SsfDevelope        decrypt data for recipient                   */
/*    SsfEncode          encode data                                  */
/*    SsfDecode          decode data                                  */
/*    SsfAddSign         add a digital signature                      */
/*    SsfDigest          compute a hash value                         */
/*    SsfDELSsfOctetstring free returned mem from the other Ssf-func  */
/*                                                                    */
/*                                                                    */
/*  Auxiliary functions:                                              */
/*                                                                    */
/*  This header file contains the function declarations for the       */
/*  auxiliary functions of the SSF ("Secure Store & Forward") API.    */
/*  The auxiliary functions can be used by applications using the     */
/*  SSF API to work with the SSF list structures which have to be     */
/*  supplied as input parameters or which are returned as output      */
/*  parameters. In particular, the auxiliary functions deal with      */
/*  security information lists for signers and recipients. The        */
/*  following auxiliary functions are provided:                       */
/*                                                                    */
/*    SsfNEWSigRcpSsfInfo     create new signer/recipient info        */
/*    SsfINSSigRcpSsfInfo     insert signer/recipient info into list  */
/*    SsfDELSigRcpSsfInfo     delete signer/recipient info            */
/*    SsfDELSigRcpSsfInfoList delete signer/recipient info list       */
/*                                                                    */
/*                                                                    */
/* Documentation:                                                     */
/*   - "Secure Store & Forward (SSF) - Spezifikation"                 */
/*                                                                    */
/*                                                                    */
/*  Author:    Michael Friedrich                                      */
/*  Reviewer:                                                         */
/*  Version:   1.50                                                   */
/*                                                                    */
/*  Description:                                                      */
/*    This file is to be included in all "C" source files that        */
/*    implement the SSF API.                                          */
/*    Sources that want to use SSF please include ssfxxall.h          */
/*                                                                    */
/*  Prerequisites:                                                    */
/*    - standard conformance: platform-independent "C" code           */
/*    - others:                                                       */
/*                                                                    */
/*  Modified:                                                         */
/*                                                                    */
/*  Remarks:                                                          */
/*                                                                    */
/**********************************************************************/

#ifndef SSFXXAPI_H
#define SSFXXAPI_H  1

#include "ssfxxall.h"

#ifdef __cplusplus
extern "C" {
#endif

/**********************************************************************/
/*                                                                    */
/*  Export interface                                                  */
/*    - Function list                                                 */
/*                                                                    */
/**********************************************************************/

#ifndef APIENTRY
#  if defined(_WINDOWS)
#    define APIENTRY        __far __export __pascal
#  elif defined(_WIN32)
#    define APIENTRY        __stdcall
#  else
#    define APIENTRY
#  endif
#endif



/*--------------------------------------------------------*/
/* SsfVersion                                             */
/*                                                        */
/* Returns version information of the SSF library:        */
/*     - version number of the SSF library                */
/*     - product name of underlying security toolkit      */
/*     - version number of underlying security toolkit    */
/*     - any additional information                       */
/*                                                        */
/* The information is concatenated into the output string */
/* using ';' as delimiter.                                */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrOutputData        version information              */
/* ostrOutputDataL       length of above                  */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK            operation successful             */
/* SSF_API_NOSECTK       security toolkit not found       */
/* SSF_API_NOMEMORY      no more memory for output data   */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfVersion (
			SsfOctetstring		* outputData,
			SsfOctetlen		* outputDataL );



/*--------------------------------------------------------*/
/* SsfQueryProperties                                     */
/*                                                        */
/* Returns properties of the SSF library:                 */
/*     - PROPERTIES	(Arg: ignored)                    */
/*     - FORMATS	(Arg: ignored)                    */
/*     - HASHALGS       (Arg optional: Format)            */
/*     - ENCRALGS       (Arg optional: Format)            */
/*                                                        */
/* The information is returned in ostrOutputData,         */
/* multple entries are separated by ';'.                  */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strProperty           property                         */
/* strPropertyL          length of strProperty            */
/*                                                        */
/* strArgument           (optional) Argument              */
/* strArgumentL          length of strArgument            */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrOutputData        version information              */
/* ostrOutputDataL       length of above                  */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK            operation successful             */
/* SSF_API_NODATA        couldn't handle this property    */
/* SSF_API_NOMEMORY      no more memory for output data   */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfQueryProperties(
			const SsfCharstring		strProperty,
			SsfCharlen				strPropertyL,
			const SsfCharstring		strArgument,
			SsfCharlen				strArgumentL,
			SsfOctetstring			* outputData,
			SsfOctetlen				* outputDataL );

/*--------------------------------------------------------*/
/* SsfEncode                                              */
/*                                                        */
/* Encodes the given data according to the specified      */
/* format.                                                */
/*                                                        */
/* Supported formats:                                     */
/*                                                        */
/* PKCS7                 Data (output format)             */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strFormat             target format                    */
/* strFormatL            length of strFormat              */
/* ostrInputData         input data to be encoded         */
/* ostrInputDataL        length of ostrInputData          */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrEncodedData       encoded data (output data)       */
/* ostrEncodedDataL      length of output data            */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK                  success for each signer    */
/* SSF_API_NOSECTK             security toolkit not found */
/* SSF_API_INVALID_FORMAT      unknown wrapper format     */
/* SSF_API_NODATA              input data has length zero */
/* SSF_API_NOMEMORY            no main memory output data */
/* SSF_API_ENCODE_FAILED        ASN.1 encoding failed     */
/* SSF_API_UNKNOWN_SECTK_ERROR unknown sec. toolk. error  */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfEncode (
			const SsfCharstring		strFormat,
			SsfCharlen				strFormatL,
			const SsfOctetstring	ostrInputData,
			SsfOctetlen				ostrInputDataL,
			SsfOctetstring			* ostrEncodedData,
			SsfOctetlen				* ostrEncodedDataL);



/*--------------------------------------------------------*/
/* SsfDecode                                              */
/*                                                        */
/* Decodes the given data according to the specified      */
/* format.                                                */
/*                                                        */
/* Supported formats:                                     */
/*                                                        */
/* PKCS7                 Data (input format)              */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strFormat             target format                    */
/* strFormatL            length of strFormat              */
/* ostrEncodedData       input data to be decoded         */
/* ostrEncodedDataL      length of ostrEncodedData        */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrOutputData        decoded data (output data)       */
/* ostrOutputDataL       length of output data            */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK                  success for each signer    */
/* SSF_API_NOSECTK             security toolkit not found */
/* SSF_API_INVALID_FORMAT      unknown wrapper format     */
/* SSF_API_NODATA              input data has length zero */
/* SSF_API_NOMEMORY            no main memory output data */
/* SSF_API_DECODE_FAILED       ASN.1 decoding failed      */
/* SSF_API_UNKNOWN_SECTK_ERROR unknown sec. toolk. error  */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfDecode (
			  const SsfCharstring	strFormat,
              SsfCharlen			strFormatL,
              const SsfOctetstring	ostrEncodedData,
		      SsfOctetlen			ostrEncodedDataL,
		      SsfOctetstring		* ostrOutputData,
			  SsfOctetlen			* ostrOutputDataL);

/*--------------------------------------------------------*/
/* SsfSign                                                */
/*                                                        */
/* Produces digital signatures for the given input data   */
/* and the given signers and returns the signed data in   */
/* the desired format.                                    */
/*                                                        */
/* Supported formats:                                     */
/*                                                        */
/* PKCS7                 SignedData (output format)       */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strFormat             target format                    */
/* strFormatL            length of strFormat              */
/* strHashalg            hash algorithm                   */
/* strHashalgL           length of strHashalg             */
/* bIncCerts             include certificates or not      */
/* bDetached             include data or not              */
/* ostrInputData         input data to be signed          */
/* ostrInputDataL        length of ostrInputData          */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* signerList            list of signers, their security  */
/*                       information and SSF result after */
/*                       function return                  */
/*                                                        */
/* SSF_API_SIGNER_OK          operation successful        */
/* SSF_API_INVALID_NAMESPACE  unknown namespace id        */
/* SSF_API_UNKNOWN_SIGNER     unknown signer              */
/* SSF_API_UNKNOWN_PROFILE    security profile not found  */
/* SSF_API_INVALID_PROFILE    security profile not usable */
/* SSF_API_INVALID_PASSWORD   invalid password for signer */
/* SSF_API_NOCERTIFICATE      certificate not found       */
/* SSF_API_SIGNER_NOT_OK      signature not ok for signer */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrSignedData        signed data (output data)        */
/* ostrSignedDataL       length of output data            */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK                  success for each signer    */
/* SSF_API_NOSECTK             security toolkit not found */
/* SSF_API_INVALID_FORMAT      unknown wrapper format     */
/* SSF_API_NODATA              input data has length zero */
/* SSF_API_NOMEMORY            no main memory output data */
/* SSF_API_SIGNER_ERRORS       there are signer errors    */
/* SSF_API_INVALID_MDALG       unknown hash algorithm     */
/* SSF_API_ENCODE_FAILED        ASN.1 encoding failed     */
/* SSF_API_DECODE_FAILED        ASN.1 decoding failed     */
/* SSF_API_UNKNOWN_SECTK_ERROR unknown sec. toolk. error  */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfSign(
			const SsfCharstring		strFormat,
			SsfCharlen				strFormatL,
			const SsfCharstring		strHashalg,
			SsfCharlen				strHashalgL,
			SAP_BOOL				bIncCerts,
			SAP_BOOL				bDetached,
			const SsfOctetstring	ostrInputData,
			SsfOctetlen				ostrInputDataL,
			SigRcpSsfInformationList signerList,
			SsfOctetstring		* ostrSignedData,
			SsfOctetlen			* ostrSignedDataL);



/*--------------------------------------------------------*/
/* SsfVerify                                              */
/*                                                        */
/* Checks digital signatures under the given input data   */
/* in the specified format and returns a list with the    */
/* verification results for each signer.                  */
/*                                                        */
/* Supported formats:                                     */
/*                                                        */
/* PKCS7                 SignedData (input format)        */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strFormat             source format                    */
/* strFormatL            length of strFormat              */
/* bUseCerts             use included certificates or not */
/* ostrSignedData        signed data (input data)         */
/* ostrSignedDataL       length of ostrSignedData         */
/* ostrInputData         data to compare with             */
/* ostrInputDataL        length of data to compare with   */
/* strPab                name of Private Address Book     */
/*                       (if any, string can be empty)    */
/* strPabL               length of strPab                 */
/* strPabPassword        password to open PAB             */
/*                       (if any, string can be empty)    */
/* strPabPasswordL       length of strPabPassword         */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* signerResultList      list of result of the operation  */
/*                       for each signer:                 */
/*                                                        */
/* SSF_API_SIGNER_OK          operation successful        */
/* SSF_API_INVALID_NAMESPACE  unknown namespace id        */
/* SSF_API_UNKNOWN_SIGNER     unknown signer              */
/* SSF_API_UNKNOWN_PROFILE    security profile not found  */
/* SSF_API_INVALID_PROFILE    security profile not usable */
/* SSF_API_INVALID_PASSWORD   invalid password for signer */
/* SSF_API_NOCERTIFICATE      certificate not found       */
/* SSF_API_SIGNER_NOT_OK      signature invalid           */
/*                                                        */
/* ostrOutputData        decoded data that was signed     */
/* ostrOutputDataL       length of ostrOutputData         */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK                   success for each signer   */
/* SSF_API_NOSECTK              security toolkit not found*/
/* SSF_API_INVALID_FORMAT       unknown wrapper format    */
/* SSF_API_NODATA               input data has length zero*/
/* SSF_API_NOMEMORY             no main memory output data*/
/* SSF_API_SIGNER_ERRORS        there are signer errors   */
/* SSF_API_NORESULTLISTMEMORY   no main memory result list*/
/* SSF_API_UNKNOWN_PAB          Priv.Addr.Book not found  */
/* SSF_API_INVALID_PAB_PASSWORD invalid PAB password      */
/* SSF_API_ENCODE_FAILED        ASN.1 encoding failed     */
/* SSF_API_DECODE_FAILED        ASN.1 decoding failed     */
/* SSF_API_UNKNOWN_SECTK_ERROR unknown sec. toolk. error  */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfVerify (
			const SsfCharstring		strFormat,
			SsfCharlen				strFormatL,
		    SAP_BOOL				bUseCerts,
			const SsfOctetstring	ostrSignedData,
		    SsfOctetlen				ostrSignedDataL,
			const SsfOctetstring	ostrInputData,
		    SsfOctetlen				ostrInputDataL,
			const SsfCharstring		strPab,
			SsfCharlen				strPabL,
			const SsfCharstring		strPabPassword,
			SsfCharlen				strPabPasswordL,
			SigRcpSsfInformationList * signerResultList,
			SsfOctetstring			* ostrOutputData,
			SsfOctetlen				* ostrOutputDataL);

/*--------------------------------------------------------*/
/* SsfEnvelope                                            */
/*                                                        */
/* Produces a digital envelope for the given input data   */
/* and the given list of recipients and returns the       */
/* enveloped data in the desired format. A list with the  */
/* operation results for each recipient is returned as    */
/* as well.                                               */
/*                                                        */
/* Supported formats:                                     */
/*                                                        */
/* PKCS7                 EnvelopedData (output format)    */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strFormat             target format                    */
/* strFormatL            length of strFormat              */
/* strSymEncr            symmetric encryption method      */
/* strSymEncrL           length of strSymEncr             */
/* ostrInputData         input data to be enveloped       */
/* ostrInputDataL        length of ostrInputData          */
/* strPab                name of Private Address Book     */
/*                       (if any, string can be empty)    */
/* strPabL               length of strPab                 */
/* strPabPassword        password to open PAB             */
/*                       (if any, string can be empty)    */
/* strPabPasswordL       length of strPabPassword         */
/*                                                        */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* recipientList         list of recipients, their        */
/*                       security information and the     */
/*                       SSF function result after return */
/*                                                        */
/* SSF_API_RECIPIENT_OK       operation successful        */
/* SSF_API_INVALID_NAMESPACE  unknown namespace id        */
/* SSF_API_UNKNOWN_RECIPIENT  unknown recipient           */
/* SSF_API_UNKNOWN_PROFILE    security profile not found  */
/* SSF_API_INVALID_PROFILE    security profile not usable */
/* SSF_API_NOCERTIFICATE      certificate not found       */
/* SSF_API_RECIPIENT_NOT_OK   operation not successful    */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrEnvelopedData     enveloped data (output data)     */
/* ostrEnvelopedDataL    length of output data            */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK                   success for each recipient*/
/* SSF_API_NOSECTK              security toolkit not found*/
/* SSF_API_INVALID_FORMAT       unknown wrapper format    */
/* SSF_API_NODATA               input data has length zero*/
/* SSF_API_NOMEMORY             no main memory output data*/
/* SSF_API_UNKNOWN_PAB          Priv.Addr.Book not found  */
/* SSF_API_INVALID_PAB_PASSWORD invalid PAB password      */
/* SSF_API_RECIPIENT_ERRORS     there are recipient errors*/
/* SSF_API_INVALID_MDALG        unknown encryption alg.   */
/* SSF_API_ENCODE_FAILED        ASN.1 encoding failed     */
/* SSF_API_DECODE_FAILED        ASN.1 decoding failed     */
/* SSF_API_UNKNOWN_SECTK_ERROR unknown sec. toolk. error  */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfEnvelope(
		const SsfCharstring		strFormat,
		SsfCharlen				strFormatL,
		const SsfCharstring		strSymEncr,
		SsfCharlen				strSymEncrL,
		const SsfOctetstring	ostrInputData,
		SsfOctetlen				ostrInputDataL,
		const SsfCharstring		strPab,
		SsfCharlen				strPabL,
		const SsfCharstring		strPabPassword,
		SsfCharlen				strPabPasswordL,
        SigRcpSsfInformationList recipientList,
		SsfOctetstring			* ostrEnvelopedData,
		SsfOctetlen				* ostrEnvelopedDataL);


/*--------------------------------------------------------*/
/* SsfDevelope                                            */
/*                                                        */
/* Removes the digital envelope from the given input data */
/* for the given recipient and returns the decrypted      */
/* output data. The operation results for the given       */
/* recipient are returned as well.                        */
/*                                                        */
/* Supported formats:                                     */
/*                                                        */
/* PKCS7                 EnvelopedData (input format)     */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strFormat             source format                    */
/* strFormatL            length of strFormat              */
/* ostrEnvelopedData     input data to be developed       */
/* ostrEnvelopedDataL    length of ostrEnvelopedData      */
/*                                                        */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* ptrRecipient          recipient, his/her security      */
/*                       information and the SSF operation*/
/*                       result after function return     */
/*                                                        */
/* SSF_API_RECIPIENT_OK       operation successful        */
/* SSF_API_INVALID_NAMESPACE  unknown namespace id        */
/* SSF_API_UNKNOWN_RECIPIENT  unknown recipient           */
/* SSF_API_UNKNOWN_PROFILE    security profile not found  */
/* SSF_API_INVALID_PROFILE    security profile not usabl  */
/* SSF_API_NOCERTIFICATE      certificate not found       */
/* SSF_API_RECIPIENT_NOT_OK   operation not successful    */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrOutputData        developed data (output data)     */
/* ostrOutputDataL       length of output data            */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK                   success for recipient     */
/* SSF_API_NOSECTK              security toolkit not found*/
/* SSF_API_INVALID_FORMAT       unknown wrapper format    */
/* SSF_API_NODATA               input data has length zero*/
/* SSF_API_NOMEMORY             no main memory output data*/
/* SSF_API_RECIPIENT_ERRORS     there are recipient errors*/
/* SSF_API_ENCODE_FAILED        ASN.1 encoding failed     */
/* SSF_API_DECODE_FAILED        ASN.1 decoding failed     */
/* SSF_API_UNKNOWN_SECTK_ERROR unknown sec. toolk. error  */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfDevelope(
			const SsfCharstring		strFormat,
			SsfCharlen				strFormatL,
			const SsfOctetstring	ostrEnvelopedData,
			SsfOctetlen				ostrEnvelopedDataL,
			PtrSigRcpSsfInformation	ptrRecipient,
			SsfOctetstring			* ostrOutputData,
			SsfOctetlen				* ostrOutputDataL);



/*--------------------------------------------------------*/
/* SsfAddSign                                             */
/*                                                        */
/* Adds another signer to already signed data.            */
/*                                                        */
/* Supported formats:                                     */
/*                                                        */
/* PKCS7                 SignedData (in/output format)    */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strFormat             target format                    */
/* strFormatL            length of strFormat              */
/* strHashalg            hash algorithm                   */
/* strHashalgL           length of strHashalg             */
/* bIncCerts             include certificates or not      */
/* ostrSignedData        signed data to be signed         */
/* ostrSignedDataL       length of ostrSignedData         */
/* ostrInputData         input data to be signed          */
/* ostrInputDataL        length of ostrInputData          */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* signer                signer information, security     */
/*                       information and SSF result after */
/*                       function return                  */
/*                                                        */
/* SSF_API_SIGNER_OK          operation successful        */
/* SSF_API_INVALID_NAMESPACE  unknown namespace id        */
/* SSF_API_UNKNOWN_SIGNER     unknown signer              */
/* SSF_API_UNKNOWN_PROFILE    security profile not found  */
/* SSF_API_INVALID_PROFILE    security profile not usable */
/* SSF_API_INVALID_PASSWORD   invalid password for signer */
/* SSF_API_NOCERTIFICATE      certificate not found       */
/* SSF_API_SIGNER_NOT_OK      signature not ok for signer */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrUpdatedSignedData        signed data (output data) */
/* ostrUpdatedSignedDataL       length of output data     */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK                  success                    */
/* SSF_API_NOSECTK             security toolkit not found */
/* SSF_API_INVALID_FORMAT      unknown wrapper format     */
/* SSF_API_NODATA              wrong input data           */
/* SSF_API_NOMEMORY            no main memory output data */
/* SSF_API_SIGNER_ERRORS       there are signer errors    */
/* SSF_API_INVALID_MDALG       unknown hash algorithm     */
/* SSF_API_ENCODE_FAILED        ASN.1 encoding failed     */
/* SSF_API_DECODE_FAILED        ASN.1 decoding failed     */
/* SSF_API_UNKNOWN_SECTK_ERROR unknown sec. toolk. error  */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfAddSign(
			const SsfCharstring		strFormat,
			SsfCharlen				strFormatL,
			const SsfCharstring		strHashalg,
			SsfCharlen				strHashalgL,
			SAP_BOOL				bIncCerts,
			const SsfOctetstring	ostrSignedData,
		    SsfOctetlen				ostrSignedDataL,
			const SsfOctetstring	ostrInputData,
		    SsfOctetlen				ostrInputDataL,
			PtrSigRcpSsfInformation	signer,
			SsfOctetstring			* ostrUpdatedSignedData,
			SsfOctetlen				* ostrUpdatedSignedDataL);



/*--------------------------------------------------------*/
/* SsfDigest                                              */
/*                                                        */
/* Computes the hash value for the given input data using */
/* the specified hash algorithm.                          */
/*                                                        */
/* Supported formats:                                     */
/*                                                        */
/* PKCS7                 Data (input format)              */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* strFormat             target format                    */
/* strFormatL            length of strFormat              */
/* bDetached             include data or not              */
/* ostrInputData         input data to be digested        */
/* ostrInputDataL        length of ostrInputData          */
/* strHashalg            hash algorithm                   */
/* strHashalgL           length of strHashalg             */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrDigestedData      digested data (output data)      */
/* ostrDigestedDataL     length of output data            */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK                  success                    */
/* SSF_API_NOSECTK             security toolkit not found */
/* SSF_API_INVALID_FORMAT      unknown wrapper format     */
/* SSF_API_NODATA              input data has length zero */
/* SSF_API_NOMEMORY            no main memory output data */
/* SSF_API_INVALID_MDALG       unknown hash algorithm     */
/* SSF_API_ENCODE_FAILED       ASN.1 encoding failed      */
/* SSF_API_DECODE_FAILED       ASN.1 decoding failed      */
/* SSF_API_UNKNOWN_SECTK_ERROR unknown sec. toolk. error  */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfDigest(
			const SsfCharstring		strFormat,
			SsfCharlen				strFormatL,
			SAP_BOOL				bDetached,
			const SsfOctetstring	ostrInputData,
			SsfOctetlen				ostrInputDataL,
			const SsfCharstring		strHashalg,
			SsfCharlen				strHashalgL,
			SsfOctetstring			* ostrDigestedData,
			SsfOctetlen				* ostrDigestedDataL);


/*--------------------------------------------------------*/
/* SsfDELSsfOctetstring                                   */
/*                                                        */
/* Deletes output data from previous SSF calls            */
/* (frees corresponding memory).                          */
/*                                                        */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* SsfOctetstring ostrToBeDeleted      data to be deleted */
/* SsfOctetlen    ostrToBeDeletedL     length of above    */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_API_OK               operation successful          */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfDELSsfOctetstring (
			SsfOctetstring	* ostrToBeDeleted,
			SsfOctetlen		* ostrToBeDeletedL );


/**********************************************************************/
/*                                                                    */
/*  AUXILIARY                                                         */
/*                                                                    */
/**********************************************************************/

/**********************************************************************/
/*                                                                    */
/*  Export interface                                                  */
/*    - Function list                                                 */
/*                                                                    */
/**********************************************************************/


/* SSF auxiliary functions for signer/recipient SSF information lists */

/*--------------------------------------------------------*/
/* SsfNEWSigRcpSsfInfo                                    */
/*                                                        */
/* Creates a new signer/recipient SSF information data    */
/* structure by allocating and initializing memory.       */
/* Returns a pointer to the new signer/recipient          */
/* information data.                                      */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* SsfCharstring  strSigRcpId           sig./recip. name  */
/* SsfCharlen     strSigRcpIdL          length of above   */
/* SsfCharstring  strSigRcpReserved	reserved for futur*/
/*                                      use (or NULL)     */
/* SsfCharlen     strSigRcpReservedL	length of above   */
/* SsfCharstring  strSigRcpProfile      sig/rcp secinfo   */
/*                                      profile name      */
/*                                      (if any, may be   */
/*                                      empty)            */
/* SsfCharlen     strSigRcpProfileL     length of above   */
/* SsfCharstring  strSigRcpPassword     password for      */
/*                                      sig/rcp secinfo   */
/*                                      (if any, may be   */
/*                                      empty)            */
/* SsfCharlen     strSigRcpPasswordL    length of above   */
/* SAP_UINT       uResult               result value      */
/*                                                        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* PtrSigRcpSsf                         ptrSigRcpSsfInfo  */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOMEMORY      insuff. main memory output data  */
/* SSF_AUX_NOSIGRCPID    sig/rcp id is empty              */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfNEWSigRcpSsfInfo(
			const SsfCharstring		strSigRcpId,
			SsfCharlen				strSigRcpIdL,
			const SsfCharstring		strSigRcpReserved,
			SsfCharlen				strSigRcpReservedL,
			const SsfCharstring		strSigRcpProfile,
			SsfCharlen				strSigRcpProfileL,
			const SsfCharstring		strSigRcpPassword,
			SsfCharlen				strSigRcpPasswordL,
			SAP_UINT				uResult,
			PtrSigRcpSsfInformation * ptrSigRcpSsfInfo);



/*--------------------------------------------------------*/
/* SsfDELSigRcpSsfInfo                                    */
/*                                                        */
/* Deletes a signer/recipient information data structure  */
/* and frees all associated memory (including contents).  */
/* Returns a NULL pointer.                                */
/*                                                        */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* PtrSigRcpSsfInformation         ptrSigRcpSsfInfo       */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOSIGRCPINFO  signer/recipient information     */
/*                       data structure is empty          */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfDELSigRcpSsfInfo  (
			PtrSigRcpSsfInformation  * ptrSigRcpSsfInfo);



/*--------------------------------------------------------*/
/* SsfINSSigRcpSsfInfo                                    */
/*                                                        */
/* Creates a new signer/recipient information list element*/
/* and inserts a given signer/recipient information data  */
/* structure at the end of a given signer/recipient       */
/* information list.                                      */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* PtrSigRcpSsfInformation        ptrSigRcpSsfInfo        */
/*                                                        */
/*                                                        */
/* INPUT/OUTPUT                                           */
/*                                                        */
/* SigRcpSsfInformationList       sigRcpSsfInfoList       */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOMEMORY      insuff. main memory output data  */
/* SSF_AUX_NOSIGRCPINFO  signer/recipient information     */
/*                       data structure is empty          */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfINSSigRcpSsfInfo  (
			const PtrSigRcpSsfInformation  ptrSigRcpSsfInfo,
			SigRcpSsfInformationList * sigRcpSsfInfoList );



/*--------------------------------------------------------*/
/* SsfDELSigRcpSsfInfoList                                */
/*                                                        */
/* Deletes a signer/recipient information list element by */
/* element and frees the corresponding memory.            */
/* (including content structure)                          */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* SigRcpSsfInformationList       sigRcpSsfInfoList       */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK               operation successful          */
/* SSF_AUX_NOSIGRCPINFOLIST signer/recipient info list is */
/*                          empty                         */
/*                                                        */
/*--------------------------------------------------------*/

SAPRETURN APIENTRY SsfDELSigRcpSsfInfoList      (
			SigRcpSsfInformationList * sigRcpSsfInfoList );



/**********************************************************/
/*                                                        */
/*  Function:  SsfPRISigRcpSsfInfo                        */
/*                                                        */
/*  Author:    M. FRIEDRICH                               */
/*  Version:   1.32                                       */
/*                                                        */
/* Prints  a signer/recipient information data structure  */
/* to memory                                              */
/*                                                        */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* PtrSigRcpSsfInformation        ptrSigRcpSsfInfo        */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrOutputData        signer/recipient information     */
/* ostrOutputDataL       length of above                  */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOMEMORY      no more memory for output data   */
/* SSF_AUX_NOSIGRCPINFO  signer/recipient information     */
/*                       data structure is empty          */
/*                                                        */
/**********************************************************/

/* NOTE: the next two functions have been removed from the API */
/*       and aren't used any more. They are left here so that  */
/*       old kernels will not core. The functions currently    */
/*       always return SSF_AUX_NOMEMORY !!!                    */
SAPRETURN APIENTRY SsfPRISigRcpSsfInfo(
				PtrSigRcpSsfInformation ptrSigRcpSsfInfo,
				SsfOctetstring		* outputData,
				SsfOctetlen			* outputDataL);

/**********************************************************/
/*                                                        */
/*  Function:  SsfPRISigRcpSsfInfoList                    */
/*                                                        */
/*  Author:    M. FRIEDRICH                               */
/*  Version:   1.32                                       */
/*                                                        */
/* Prints  a signer/recipient information list element by */
/* element to memory                                      */
/*                                                        */
/* INPUT                                                  */
/*                                                        */
/* SigRcpSsfInformationList       sigRcpSsfInfoList       */
/*                                                        */
/* OUTPUT                                                 */
/*                                                        */
/* ostrOutputData        list information                 */
/* ostrOutputDataL       length of above                  */
/*                                                        */
/* RETURN                                                 */
/*                                                        */
/* SSF_AUX_OK            operation successful             */
/* SSF_AUX_NOMEMORY      no more memory for output data   */
/* SSF_AUX_NOSIGRCPINFOLIST signer/recipient info list is */
/*                          empty                         */
/*                                                        */
/**********************************************************/


SAPRETURN APIENTRY SsfPRISigRcpSsfInfoList(
				SigRcpSsfInformationList sigRcpSsfInfoList,
				SsfOctetstring		* outputData,
				SsfOctetlen			* outputDataL);


#ifdef __cplusplus
}
#endif

#endif  /* SSFXXAPI_H */
